---- Mod settings interface
-- @author Oskar Eisemuth
-- @copyright 2017-2019
-- @module commonapi.settings
local settings = {
	_settingscache = {}
}

local _log = commonapi.utils.logPrep("commonapi.settings")

local function readUserSettings(filename)
	local luafnSettings, err
	local result
	local env = {}
	setmetatable(env, {__index=_G})
	luafnSettings, err = loadfile(filename, "bt", env)
	if (luafnSettings == nil) then
		return {}
	end
	result = luafnSettings()
	if (type(result) ~= "table") then
		_log("Result of "..filename.. " not a table?!")
		result = {}
	end
	return result
end
	
local validateDefaults = {
	["number"] = function() return 0 end,
	["string"] = function() return "" end,
	["boolean"] = function() return false end,
	["table"] = function() return {} end,
}
local function validateSettings(defsettings, usersettings)
	local result = {}
	if (type(usersettings) ~= "table") then
		usersettings = {}
	end
	
	for key, def in pairs(defsettings) do
		local uservalue = usersettings[key]
		if type(uservalue) == def.type then
			result[key] = uservalue
		else
			if (def.default ~= nil) then
				result[key] = def.default
			elseif (validateDefaults[def.type] ~= nil) then
				result[key] = validateDefaults[def.type]()
			end
		end
	end
	
	return result
end


function settings._getFromEntry(entry)
	local result = {}
	if (entry ~= nil and entry.path ~= nil and entry.data ~= nil) then
		local defsettings = entry.data.settings or {}
		if (defsettings ~= nil) then
			result = validateSettings(defsettings, readUserSettings(entry.path .. "settings.lua"))
		end
	end
	return result
end


--- Get settings for modid (provided by settings.lua of mod)
-- @tparam string modid
-- @tparam[opt] string key to get only specific setting or nil
-- @return[1] A table of settings
-- @return[2] Setting value, default value or nil if key not found
function settings.get(modid, key)
	local result = {}
	if (settings._settingscache[modid] ~= nil) then
		result = settings._settingscache[modid]
	else
		local entry = commonapi.mods.getById(modid)
		result = settings._getFromEntry(entry)
		settings._settingscache[modid] = result
	end

	if (key ~= nil) then
		return result[key]
	end
	return result
end

function settings._clearCache(modid)
	if (modid == nil) then
		settings._settingscache = {}
	else
		settings._settingscache[modid] = nil
	end
end


function settings._getDescriptionFromEntry(entry)
	if (entry ~= nil and entry.data ~= nil) then
		if (type(entry.data.settings) == 'table') then
			return entry.data.settings
		end
	end
	return nil
end

function settings._getDescription(modid)
	local entry = commonapi.mods.getById(modid)
	return settings._getDescriptionFromEntry(entry)
end



return settings